@@ -1,6 +1,7 @@
 import os
 import random
 import numpy as np
+import re
 from getdist.paramnames import ParamNames, ParamInfo, escapeLatex
 from getdist.convolve import autoConvolve
 from getdist import cobaya_interface
@@ -87,24 +88,22 @@ def chainFiles(root, chain_indices=None, ext='.txt', separator="_",
     :param chain_exclude: A list of indexes to exclude, None to include all
     :return: The list of file names
     """
-    index = -1
+
+    folder = os.path.dirname(root)
+    if root.endswith((os.sep, "/")):
+        reg_exp = re.compile('(?P<num>[0-9]+)?' + re.escape(ext))
+    else:
+        basename = os.path.basename(root)
+        reg_exp = re.compile(re.escape(basename) + '(' + re.escape(separator) + '(?P<num>[0-9]+))?' + re.escape(ext))
     files = []
-    while True:
-        index += 1
-        fname = root
-        if index > 0:
-            # deal with just-folder prefix
-            if not root.endswith((os.sep, "/")):
-                fname += separator
-            fname += str(index)
-        if not fname.endswith(ext):
-            fname += ext
-        if index > first_chain and not os.path.exists(fname) or 0 < last_chain < index:
-            break
-        if (chain_indices is None or index in chain_indices) \
-                and (chain_exclude is None or index not in chain_exclude) \
-                and index >= first_chain and os.path.exists(fname):
-            files.append(fname)
+    for f in os.listdir(folder):
+        match = reg_exp.fullmatch(f)
+        if match:
+            index = int(match.group("num") or 0)
+            if (chain_indices is None or index in chain_indices) \
+                    and (chain_exclude is None or index not in chain_exclude) \
+                    and index >= first_chain and (last_chain < 0 or index <= last_chain):
+                files.append(os.path.join(folder, f))
     return files
 
 
@@ -1002,9 +1001,11 @@ class WeightedSamples:
         fixed = []
         values = []
         for i in range(self.samples.shape[1]):
-            if np.all(self.samples[:, i] == self.samples[0, i]):
-                fixed.append(i)
-                values.append(self.samples[0, i])
+            if np.isclose(self.samples[0, i], self.samples[-1, i]):
+                mean = np.average(self.samples[:, i])
+                if np.allclose(self.samples[:, i], mean, rtol=1e-12, atol=0):
+                    fixed.append(i)
+                    values.append(mean)
         self.changeSamples(np.delete(self.samples, fixed, 1))
         return fixed, values
 
@@ -1073,7 +1074,7 @@ class Chains(WeightedSamples):
         """
 
         self.chains = None
-        WeightedSamples.__init__(self, **kwargs)
+        super().__init__(**kwargs)
         self.jobItem = jobItem
         self.ignore_lines = float(kwargs.get('ignore_rows', 0))
         self.root = root
@@ -1237,11 +1238,11 @@ class Chains(WeightedSamples):
         :return: ordered dictionary of parameter values
         """
         res = dict()
+        res['weight'] = self.weights[ix]
+        res['loglike'] = self.loglikes[ix]
         for i, name in enumerate(self.paramNames.names):
             if want_derived or not name.isDerived:
                 res[name.name] = self.samples[ix, i]
-        res['weight'] = self.weights[ix]
-        res['loglike'] = self.loglikes[ix]
         return res
 
     def _makeParamvec(self, par):
@@ -1251,7 +1252,7 @@ class Chains(WeightedSamples):
             par = par.name
         if isinstance(par, str):
             return self.samples[:, self.index[par]]
-        return WeightedSamples._makeParamvec(self, par)
+        return super()._makeParamvec(par)
 
     def updateChainBaseStatistics(self):
         # old name, use updateBaseStatistics
@@ -1451,17 +1452,11 @@ class Chains(WeightedSamples):
         Delete parameters that are fixed (the same value in all samples)
         """
         if self.samples is not None:
-            fixed, values = WeightedSamples.deleteFixedParams(self)
+            fixed, values = super().deleteFixedParams()
             self.chains = None
         else:
-            fixed = []
-            values = []
-            chain = self.chains[0]
-            for i in range(chain.n):
-                if np.all(chain.samples[:, i] == chain.samples[0, i]):
-                    fixed.append(i)
-                    values.append(chain.samples[0, i])
-            for chain in self.chains:
+            fixed, values = self.chains[0].deleteFixedParams()
+            for chain in self.chains[1:]:
                 chain.changeSamples(np.delete(chain.samples, fixed, 1))
         if hasattr(self, 'ranges'):
             for ix, value in zip(fixed, values):
